package de.intevation.flys.exports;

import org.apache.log4j.Logger;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

import de.intevation.artifacts.Artifact;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.WINFOArtifact;

import de.intevation.flys.artifacts.model.FacetTypes;


/**
 * An OutGenerator that generates cross section graphs.
 */
public class CrossSectionGenerator
extends      XYChartGenerator
implements   FacetTypes
{
    /** The logger that is used in this generator. */
    private static Logger logger =
        Logger.getLogger(CrossSectionGenerator.class);

    public static final String I18N_CHART_TITLE =
        "chart.cross_section.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.cross_section.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.cross_section.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.cross_section.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT = "Querprofildiagramm";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Abstand [m]";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "W [NN + m]";


    /** Trivial Constructor. */
    public CrossSectionGenerator() {
        super();
    }


    /**
     * Get localized chart title.
     */
    protected String getChartTitle() {
        // TODO get river etc for localized heading
        Object[] i18n_msg_args = new Object[] {
            getRiverName()
        };
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT, i18n_msg_args);
    }


    /**
     * Add localized subtitle to chart.
     */
    @Override
    protected void addSubtitles(JFreeChart chart) {
        double[] dist = getRange();

        Object[] args = new Object[] {
            getRiverName(),
            getKm()
        };

        String subtitle = msg(I18N_CHART_SUBTITLE, "", args);
        chart.addSubtitle(new TextTitle(subtitle));
    }


    /**
     * Get localized X Axis label.
     */
    protected String getXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }


    /**
     * Get cross_section.km data from artifact.
     */
    protected Double getKm() {
        try {
            WINFOArtifact winfo = (WINFOArtifact) master;
            return winfo.getCrossSectionSnapKm();
        }
        catch (Exception e) {
            logger.error("Cannot convert cross_section.km to double");
            return 0.0d;
        }
    }


    /**
     * Get localized Y Axis label.
     */
    protected String getYAxisLabel() {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }


    protected void adjustAxes(XYPlot plot) {
        super.adjustAxes(plot);

        NumberAxis qAxis = new NumberAxis(
            msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT));

        plot.setRangeAxis(1, qAxis);
    }


    /**
     * Let one facet do its job.
     */
    public void doOut(
        Artifact artifact,
        Facet    facet,
        Document attr,
        boolean  visible
    ) {
        String name = facet.getName();

        logger.debug("CrossSectionGenerator.doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        FLYSArtifact flys = (FLYSArtifact) artifact;
        Facet        f    = flys.getNativeFacet(facet);

        if (f == null) {
            return;
        }

        if (name.equals(CROSS_SECTION)) {
            doCrossSectionOut(
                f.getData(artifact, context),
                f.getDescription(),
                attr,
                visible);
        }
        else if (name.equals(CROSS_SECTION_WATER_LINE)) {
            doCrossSectionWaterLineOut(
                f.getData(artifact, context),
                f.getDescription(),
                attr,
                visible);
        }
        else {
            logger.warn("CrossSection.doOut: Unknown facet name: " + name);
            return;
        }
    }


    /**
     * Do cross sections waterline out.
     *
     * @param seriesName name of the data (line) to display in legend.
     * @param theme Theme for the data series.
     */
    protected void doCrossSectionWaterLineOut(
        Object     o,
        String     seriesName,
        Document   theme,
        boolean    visible
    ) {
        logger.debug("CrossSectionGenerator.doCrossSectionWaterLineOut");

        XYSeries series = new StyledXYSeries(seriesName, theme);

        double[][] a = (double [][]) o;
        double [] pxs = a[0];
        for (int i = 0; i < pxs.length; i++) {
            series.add (a[0][i], a[1][i]);
        }
        addFirstAxisSeries(series, visible);
    }


    /**
     * Do cross sections out.
     *
     * @param seriesName name of the data (line) to display in legend.
     * @param theme Theme for the data series.
     */
    protected void doCrossSectionOut(
        Object     o,
        String     seriesName,
        Document   theme,
        boolean    visible
    ) {
        logger.debug("CrossSectionGenerator.doCrossSectionOut");

        XYSeries series = new StyledXYSeries(seriesName, theme);

        double[][] a = (double [][]) o;
        double [] pxs = a[0];
        for (int i = 0; i < pxs.length; i++) {
            series.add (a[0][i], a[1][i]);
        }
        addFirstAxisSeries(series, visible);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
